<?php

namespace Drupal\vgwort\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure VG Wort settings for this site.
 *
 * @todo Consider adding validation for publisher_id and domain.
 *
 * @internal
 */
class SettingsForm extends ConfigFormBase {

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entityTypeManager) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );

  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'vgwort_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['vgwort.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('vgwort.settings');

    // Use config with overrides so any settings.php overrides are respected.
    if ($this->configFactory->get('vgwort.settings')->get('test_mode')) {
      $this->messenger()->addWarning($this->t('The test mode is enabled. The 1x1 pixel will be added as HTML comment to the selected entity_types.'));
    }

    $form['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('VG Wort username'),
      '#default_value' => $config->get('username'),
      '#description' => $this->t('Your login for VG Wort.'),
    ];

    $form['password'] = [
      '#type' => 'password',
      '#title' => $this->t('VG Wort password'),
      '#default_value' => $config->get('password'),
    ];

    $current_password = $config->get('password');
    if (!empty($current_password)) {
      $form['password']['#placeholder'] = $this->t('** Hidden for security reasons **');
    }
    $form['delete_password'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Delete the stored password'),
      '#access' => !empty($current_password),
    ];

    $form['publisher_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Publisher ID (Karteinummer)'),
      '#default_value' => $config->get('publisher_id'),
      '#description' => $this->t('Your ID provided by VG Wort.'),
    ];

    $form['image_domain'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Counter domain'),
      '#default_value' => $config->get('image_domain'),
      '#description' => $this->t('The counter domain that is provided by VG Wort.'),
    ];

    // Entity types marked as "internal" are removed from the list.
    $entity_types = array_map(
      function (EntityTypeInterface $entity_type) {
        return $entity_type->getLabel();
      },
      array_filter($this->entityTypeManager->getDefinitions(), function (EntityTypeInterface $entity_type) {
        return !$entity_type->isInternal() && $entity_type instanceof ContentEntityTypeInterface && $entity_type->entityClassImplements(EntityPublishedInterface::class);
      })
    );

    $default_value = $config->get('entity_types');
    asort($entity_types);

    $form['entity_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Entity types'),
      '#default_value' => $default_value,
      '#options' => $entity_types,
      '#description' => $this->t('Select entity types that can have a counter ID.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getValue('delete_password')) {
      $form_state->setValue('password', '');
    }
    elseif (empty($form_state->getValue('password'))) {
      // Ensure that the password is not delete unexpectedly.
      $form_state->setValue('password', $this->config('vgwort.settings')->get('password'));
    }
    $form_state->unsetValue('delete_password');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('vgwort.settings')
      ->set('username', $form_state->getValue('username'))
      ->set('password', $form_state->getValue('password'))
      ->set('publisher_id', $form_state->getValue('publisher_id'))
      ->set('image_domain', $form_state->getValue('image_domain'))
      ->set('entity_types', array_keys(array_filter($form_state->getValue('entity_types'))))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
